package org.springblade.common.notify.rocketmq.producer;

import com.alibaba.fastjson.JSONObject;
import com.aliyun.openservices.ons.api.*;
import org.springblade.common.notify.constant.NotifyErrorConstants;
import com.wn.common.validator.Validator;
import lombok.extern.slf4j.Slf4j;

import java.nio.charset.StandardCharsets;
import java.util.Properties;

/**
 * @author Cai Yihua
 **/
@Slf4j
public class AliyunProducerProxy implements ProducerProxy {
    /**
     * 真实阿里云消息生产者对象
     */
    private Producer producer;

    @Override
    public void startup(Properties properties) {
        producer = ONSFactory.createProducer(properties);
        log.info("Aliyun Producer started! Producer: {}", producer.toString());
        // 在发送消息前，必须调用 start 方法来启动 Producer，只需调用一次即可。
        producer.start();
    }

    @Override
    public void shutdown() {
        if (producer != null) {
            producer.shutdown();
            producer = null;
        }
    }

    @Override
    public boolean isStarted() {
        if (producer != null) {
            return producer.isStarted();
        }
        return false;
    }

    @Override
    public SendMqMessageResult sendMessage(String topic, String tag, String messageBody, Properties userProperties, Long startDeliverTime) {
        Validator.notEmpty(topic, NotifyErrorConstants.MQ_MESSAGE_PARMETER_ERROR);
        Validator.notEmpty(tag, NotifyErrorConstants.MQ_MESSAGE_PARMETER_ERROR);

        Message msg = new Message(
                // 您在控制台创建的 Topic
                topic,
                // Message Tag, 可理解为 Gmail 中的标签，对消息进行再归类，方便 Consumer 指定过滤条件在 MQ 服务器过滤
                tag,
                // Message Body 可以是任何二进制形式的数据， MQ 不做任何干预，需要 Producer 与 Consumer 协商好一致的序列化和反序列化方式
                messageBody.getBytes(StandardCharsets.UTF_8));
        // 设置代表消息的业务关键属性，请尽可能全局唯一。
        // 以方便您在无法正常收到消息情况下，可通过 MQ 控制台查询消息并补发。
        // 注意：不设置也不会影响消息正常收发
        msg.setKey(topic + "_" + tag + "_" + System.currentTimeMillis() / 1000);
        if (userProperties != null) {
            msg.setUserProperties(userProperties);
        }
        // 设置消息需要被投递的时间
        if (startDeliverTime != null) {
            msg.setStartDeliverTime(startDeliverTime);
        }

        SendResult sendResult = producer.send(msg);
        // 同步发送消息，只要不抛异常就是成功
        if (sendResult != null) {
            log.info("Send mq message success. Topic is: {}, tag is: {}, msgId is: {}. ", topic, tag, sendResult.getMessageId());
            SendMqMessageResult result = new SendMqMessageResult();
            result.setMessageId(sendResult.getMessageId());
            result.setOriginResult(JSONObject.toJSONString(sendResult));
            return result;
        }

        log.error("Send mq message failed. Topic is: {}, tag is: {}, message body is: {} ", topic, tag, messageBody);
        return null;
    }

    @Override
    public SendMqMessageResult sendAsyncMessage(String topic, String tag, String messageBody, Properties userProperties) {
        Validator.notEmpty(topic, NotifyErrorConstants.MQ_MESSAGE_PARMETER_ERROR);
        Validator.notEmpty(tag, NotifyErrorConstants.MQ_MESSAGE_PARMETER_ERROR);

        Message msg = new Message(
                // 您在控制台创建的 Topic
                topic,
                // Message Tag, 可理解为 Gmail 中的标签，对消息进行再归类，方便 Consumer 指定过滤条件在 MQ 服务器过滤
                tag,
                // Message Body 可以是任何二进制形式的数据， MQ 不做任何干预，需要 Producer 与 Consumer 协商好一致的序列化和反序列化方式
                messageBody.getBytes(StandardCharsets.UTF_8));
        // 设置代表消息的业务关键属性，请尽可能全局唯一。
        // 以方便您在无法正常收到消息情况下，可通过 MQ 控制台查询消息并补发。
        // 注意：不设置也不会影响消息正常收发
        msg.setKey(topic + "_" + tag + "_" + System.currentTimeMillis() / 1000);
        if (userProperties != null) {
            msg.setUserProperties(userProperties);
        }

        // 异步发送消息, 发送结果通过 callback 返回给客户端。
        SendMqMessageResult result = new SendMqMessageResult();
        producer.sendAsync(msg, new SendCallback() {
            @Override
            public void onSuccess(final SendResult sendResult) {
                // 消息发送成功
                log.info("Send mq message success. Topic is: {}, tag is: {}, msgId is: {}. ", topic, tag, sendResult.getMessageId());
                result.setMessageId(sendResult.getMessageId());
                result.setOriginResult(JSONObject.toJSONString(sendResult));
            }

            @Override
            public void onException(OnExceptionContext context) {
                // 消息发送失败，需要进行重试处理，可重新发送这条消息或持久化这条数据进行补偿处理
                log.warn("send message failed. topic: " + context.getTopic() + ", msgId: " + context.getMessageId());
                sendMessage(topic, messageBody, tag, userProperties, null);
            }
        });

        // 在 callback 返回之前即可取得 msgId
        log.info("send message async. topic: " + msg.getTopic() + ", msgId: " + msg.getMsgID());
        return result;
    }
}
